import json
from typing import List, Dict, Any, Optional, Union, Iterator, AsyncIterator
import httpx
from ...errors import APIConnectionError

class Completions:
    def __init__(self, client):
        self._client = client

    def create(
        self,
        model: str,
        messages: List[Dict[str, str]],
        stream: bool = False,
        memory: bool = False,
        memory_path: Optional[str] = None,
        **kwargs
    ) -> Union[Dict[str, Any], Iterator[Dict[str, Any]]]:
        """
        Create a chat completion.
        
        Args:
            model: The ID of the model to use.
            messages: A list of messages comprising the conversation.
            stream: If True, partial message deltas will be sent as SSE.
            memory: If True, enables conversation memory.
            memory_path: Optional path for file-based memory.
            **kwargs: Additional parameters to pass to the API.
        """
        url = "/v1/chat/completions"
        headers = self._client._get_headers(memory=memory, memory_path=memory_path)
        data = {
            "model": model,
            "messages": messages,
            "stream": stream,
            **kwargs
        }

        if stream:
            return self._stream_request(url, headers, data)
        
        try:
            response = self._client.client.post(url, headers=headers, json=data)
            return self._client._handle_response(response)
        except httpx.RequestError as e:
            raise APIConnectionError(f"Error connecting to API: {str(e)}")

    def _stream_request(self, url: str, headers: Dict[str, str], data: Dict[str, Any]) -> Iterator[Dict[str, Any]]:
        try:
            with self._client.client.stream("POST", url, headers=headers, json=data) as response:
                if not response.is_success:
                    self._client._handle_response(response)
                
                for line in response.iter_lines():
                    if line.startswith("data: "):
                        content = line[6:].strip()
                        if content == "[DONE]":
                            break
                        try:
                            yield json.loads(content)
                        except json.JSONDecodeError:
                            continue
        except httpx.RequestError as e:
            raise APIConnectionError(f"Error connecting to API: {str(e)}")

class AsyncCompletions:
    def __init__(self, client):
        self._client = client

    async def create(
        self,
        model: str,
        messages: List[Dict[str, str]],
        stream: bool = False,
        memory: bool = False,
        memory_path: Optional[str] = None,
        **kwargs
    ) -> Union[Dict[str, Any], AsyncIterator[Dict[str, Any]]]:
        """
        Create a chat completion asynchronously.
        """
        url = "/v1/chat/completions"
        headers = self._client._get_headers(memory=memory, memory_path=memory_path)
        data = {
            "model": model,
            "messages": messages,
            "stream": stream,
            **kwargs
        }

        if stream:
            return self._stream_request(url, headers, data)
        
        try:
            response = await self._client.client.post(url, headers=headers, json=data)
            return self._client._handle_response(response)
        except httpx.RequestError as e:
            raise APIConnectionError(f"Error connecting to API: {str(e)}")

    async def _stream_request(self, url: str, headers: Dict[str, str], data: Dict[str, Any]) -> AsyncIterator[Dict[str, Any]]:
        try:
            async with self._client.client.stream("POST", url, headers=headers, json=data) as response:
                if not response.is_success:
                    self._client._handle_response(response)
                
                async for line in response.aiter_lines():
                    if line.startswith("data: "):
                        content = line[6:].strip()
                        if content == "[DONE]":
                            break
                        try:
                            yield json.loads(content)
                        except json.JSONDecodeError:
                            continue
        except httpx.RequestError as e:
            raise APIConnectionError(f"Error connecting to API: {str(e)}")



class MineAIError(Exception):
    """Base class for all MineAI SDK errors."""
    def __init__(self, message: str, status_code: int = None, response: any = None):
        super().__init__(message)
        self.status_code = status_code
        self.response = response

class AuthenticationError(MineAIError):
    """Raised when the API key is invalid or missing."""
    pass

class BadRequestError(MineAIError):
    """Raised when the request is invalid."""
    pass

class RateLimitError(MineAIError):
    """Raised when the rate limit is exceeded."""
    pass

class InternalServerError(MineAIError):
    """Raised when the server encounters an error."""
    pass

class APIConnectionError(MineAIError):
    """Raised when there is a problem connecting to the API."""
    pass



from .client import MineAI, AsyncMineAI
from .models import Models
from .errors import (
    MineAIError,
    AuthenticationError,
    BadRequestError,
    RateLimitError,
    InternalServerError,
    APIConnectionError
)

__all__ = [
    "MineAI",
    "AsyncMineAI",
    "Models",
    "MineAIError",
    "AuthenticationError",
    "BadRequestError",
    "RateLimitError",
    "InternalServerError",
    "APIConnectionError",
]



import httpx
from typing import Optional, Dict, Any
from .errors import (
    AuthenticationError,
    BadRequestError,
    RateLimitError,
    InternalServerError,
    APIConnectionError,
    MineAIError
)

class BaseClient:
    def __init__(
        self,
        api_key: str,
        base_url: str = "https://studio.getmineai.site",
        timeout: float = 60.0,
    ):
        if not api_key:
            raise AuthenticationError("API key is required")
        
        self.api_key = api_key
        self.base_url = base_url.rstrip("/")
        self.timeout = timeout

    def _get_headers(self, memory: bool = False, memory_path: Optional[str] = None) -> Dict[str, str]:
        headers = {
            "Authorization": f"Bearer {self.api_key}",
            "Content-Type": "application/json",
        }
        if memory:
            headers["Memory"] = "true"
        if memory_path:
            headers["Memory-Path"] = memory_path
        return headers

    def _handle_response(self, response: httpx.Response) -> Any:
        if response.is_success:
            try:
                return response.json()
            except Exception:
                return response.text

        status_code = response.status_code
        try:
            error_data = response.json()
            message = error_data.get("error", response.text)
        except Exception:
            message = response.text

        if status_code == 401:
            raise AuthenticationError(message, status_code, response)
        elif status_code == 400:
            raise BadRequestError(message, status_code, response)
        elif status_code == 429:
            raise RateLimitError(message, status_code, response)
        elif status_code >= 500:
            raise InternalServerError(message, status_code, response)
        else:
            raise MineAIError(message, status_code, response)

class MineAI(BaseClient):
    def __init__(
        self,
        api_key: str,
        base_url: str = "https://api.mineai.dev",
        timeout: float = 60.0,
    ):
        super().__init__(api_key, base_url, timeout)
        self.client = httpx.Client(base_url=self.base_url, timeout=self.timeout)
        
        # Resources
        from .resources.chat.completions import Completions
        self.chat = Chat(self)

class Chat:
    def __init__(self, client: MineAI):
        from .resources.chat.completions import Completions
        self.completions = Completions(client)

class AsyncMineAI(BaseClient):
    def __init__(
        self,
        api_key: str,
        base_url: str = "https://studio.getmineai.site",
        timeout: float = 60.0,
    ):
        super().__init__(api_key, base_url, timeout)
        self.client = httpx.AsyncClient(base_url=self.base_url, timeout=self.timeout)
        
        # Resources
        from .resources.chat.completions import AsyncCompletions
        self.chat = AsyncChat(self)

class AsyncChat:
    def __init__(self, client: AsyncMineAI):
        from .resources.chat.completions import AsyncCompletions
        self.completions = AsyncCompletions(client)
